// keys.h - header file for reusable interface

// Abstracted pointer
typedef void * HKEYLIST;

//
// Retrieves and decodes inbound form data.  Builds list of keys, and
// pointers to data within a content file.  Returns handle to first
// element in the list.
//

HKEYLIST GetKeyList (EXTENSION_CONTROL_BLOCK *pECB);

//
// GetKeyInfo extracts the key name and content file values from the
// supplied key, and returns a handle to the next key in the list
//

HKEYLIST GetKeyInfo (HKEYLIST hKey, LPCTSTR *plpszKeyName, 
                     LPDWORD pdwOffset, LPDWORD pdwLength,
                     BOOL *pbHasCtrlChars, LPINT pnInstance);


//
// FindKey sequentially searches the linked list for a specific key.
// The return handle can be used with GetKeyInfo to get more details.
// FindKey returns the very first occurance of a duplicate key.
//

HKEYLIST FindKey (HKEYLIST hKeyList, LPCTSTR lpszSearchName);


//
// FreeKeyList releases all of the memory associated with a key list.
// Also, the content file is deleted. Call this with a handle to the 
// first element in the list.
//

void FreeKeyList (HKEYLIST hKeyList);

//
// GetContentPath returns the name of the content file associated
// with a key list.  The name is a full path, suitable for CreateFile.
//

LPCTSTR GetContentPath (HKEYLIST hKeyList);
